#!/bin/bash

SRCDIR=/tmp/hello
APPDIR=/tmp/hello.AppDir

# Halt on errors
set -e

# Assert we're on CentOS 6
grep -r "CentOS release 6" /etc/redhat-release || exit 1 

# Create the AppDir
mkdir -p $APPDIR/usr/lib

# Make sure lib64 points to lib
pushd $APPDIR/usr > /dev/null
ln -s lib lib64
popd > /dev/null

# Build the application
mkdir $SRCDIR/build
pushd $SRCDIR/build > /dev/null
cmake3 \
    -DCMAKE_INSTALL_PREFIX=$APPDIR/usr \
    -DCMAKE_BUILD_TYPE=Debug \
    ..
cmake3 --build . -- -j6
cmake3 --build . --target install
popd > /dev/null

required_libs=$(ldd $APPDIR/usr/bin/hello | grep "=>" | grep -v linux-vdso.so.1 | awk '{print $3}')
excluded_libs=$(< $SRCDIR/excluded)
assumed_pkg=$(< $SRCDIR/packages)

echo "Bundling libraries..."
for lib in $required_libs; do
    if basename $lib | grep -qxF "$excluded_libs"; then
        echo "Skipping $lib (assumed part of base system)"
    elif rpm -qf --queryformat "%{NAME}" $lib | grep -qxF "$assumed_pkg"; then
        echo "Skipping $lib (installed by $(rpm -qf $lib))"
    else
        cp $lib $APPDIR/usr/lib/
    fi
done

echo "Bundling Qt plugins..."
mkdir $APPDIR/usr/lib/qt5
cp -r /usr/lib64/qt5/plugins $APPDIR/usr/lib/qt5/

echo "Bundling libraries required by Qt plugins..."
plugins=$(find $APPDIR/usr/lib/qt5/plugins -type f -executable)
plugins_libs=$(ldd $plugins | grep "=>" | grep -v linux-vdso.so.1 | awk '{print $3}' | sort | uniq)
for lib in $plugins_libs; do
    libname=$(basename $lib)
    if [ ! -f $APPDIR/usr/lib/$libname ]; then
        if echo $libname | grep -qxF "$excluded_libs"; then
            echo "Skipping $lib (assumed part of base system)"
        elif rpm -qf --queryformat "%{NAME}" $lib | grep -qxF "$assumed_pkg"; then
            echo "Skipping $lib (installed by $(rpm -qf $lib))"
        else
            cp $lib $APPDIR/usr/lib/
        fi
    else
        echo "Skipping $lib (already installed)"
    fi
done

for file in $(find $APPDIR/usr -type f); do
    if grep -q "/usr" $file; then
        echo "WARNING: $file has references to /usr"
    fi
done

echo "Creating AppImage..."
cp /tmp/AppImageKit/AppRun $APPDIR
cp $APPDIR/usr/share/applications/hello.desktop $APPDIR
/tmp/AppImageKit/AppImageAssistant.AppDir/package $APPDIR /tmp/hello.appimage
